"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const injectable_decorator_1 = require("../decorators/services/injectable.decorator");
const logger_1 = require("../services/logger");
let Injector = Injector_1 = class Injector {
    constructor(logger) {
        this.logger = logger;
        this.temporaryValue = Symbol.for('recursive-injection');
        this.cache = new Map();
        this.cache.set(logger_1.Logger, () => this.logger);
        this.cache.set(Injector_1, () => this);
    }
    resolveInjectable(ctorFn) {
        if (!ctorFn) {
            this.logger.error('injector', 'Attempted to inject a falsey type.');
            return;
        }
        this.logger.verbose('injector', `Resolving ${ctorFn.name || ctorFn}`);
        if (this.cache.get(ctorFn) === this.temporaryValue) {
            this.logger.error('injector', `Detected circular dependency. Recursive injection of type ${ctorFn.name || ctorFn}`);
            return;
        }
        if (this.cache.has(ctorFn)) {
            return this.cache.get(ctorFn)();
        }
        this.cache.set(ctorFn, this.temporaryValue);
        let inst = this.construct(ctorFn);
        this.cache.set(ctorFn, () => inst);
        return inst;
    }
    construct(ctorFn) {
        let types = Reflect.getOwnMetadata('design:paramtypes', ctorFn);
        if (!types)
            types = [];
        let typesStr = `[${types.map(type => (type && type.name) || type).join(', ')}]`;
        this.logger.verbose('injector', `Constructing ${ctorFn.name || ctorFn} with types ${typesStr}`);
        if (types.find(type => type == Object))
            throw new Error(`Could not dependency inject ${ctorFn.name || ctorFn}. Failed to resolve constructor types. Reflected: ${typesStr}`);
        let values = types.map(type => this.resolveInjectable(type));
        return new ctorFn(...values);
    }
    provide(provideMeta) {
        if (!provideMeta)
            throw new Error(`Invalid ProvideMetadata: ${provideMeta}`);
        let ctorFn = provideMeta.provide;
        if (!ctorFn)
            throw new Error('Attempted to provide a value for a falsey type.');
        if (this.cache.has(ctorFn))
            throw new Error(`Duplicate value provided for ${ctorFn.name || ctorFn}.`);
        let tFn;
        if (this.isClassSource(provideMeta)) {
            this.logger.verbose('injector', `Providing ${ctorFn.name || ctorFn} using class ${provideMeta.useClass.name || provideMeta.useClass}`);
            let t = this.construct(provideMeta.useClass);
            tFn = () => t;
        }
        else if (this.isValueSource(provideMeta)) {
            this.logger.verbose('injector', `Providing ${ctorFn.name || ctorFn} using value ${JSON.stringify(provideMeta.useValue)}`);
            let t = provideMeta.useValue;
            tFn = () => t;
        }
        else if (this.isCallbackSource(provideMeta)) {
            this.logger.verbose('injector', `Providing ${ctorFn.name || ctorFn} using callback ${provideMeta.useCallback.name || provideMeta.useCallback}`);
            tFn = provideMeta.useCallback;
        }
        else
            throw new Error(`Could not resolve dependency injection provider source.`);
        this.cache.set(ctorFn, tFn);
        return this;
    }
    isClassSource(meta) {
        return !!meta.useClass;
    }
    isValueSource(meta) {
        return typeof meta.useValue !== 'undefined';
    }
    isCallbackSource(meta) {
        return !!meta.useCallback;
    }
};
Injector = Injector_1 = __decorate([
    injectable_decorator_1.Injectable(),
    __metadata("design:paramtypes", [logger_1.Logger])
], Injector);
exports.Injector = Injector;
var Injector_1;
//# sourceMappingURL=injector.js.map