"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const injectable_decorator_1 = require("../decorators/services/injectable.decorator");
const orm_1 = require("../metadata/server/orm");
const logger_1 = require("../services/logger");
const cls_namespace_service_1 = require("../services/cls-namespace.service");
const transaction_impl_1 = require("./impl/transaction-impl");
const __Sequelize = require("sequelize");
let Sequelize = class Sequelize {
    constructor(ormMeta, logger, namespace) {
        this.ormMeta = ormMeta;
        this.logger = logger;
        this.namespace = namespace;
        this._initialized = false;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._initialized)
                return;
            this._initialized = true;
            let orm = this.ormMeta;
            if (!orm.enabled || !orm.db)
                return;
            let db = orm.db;
            this.sql = new __Sequelize(db.name, db.user, db.password, {
                host: db.host.domain,
                port: db.host.port,
                dialect: db.dialect,
                dialectOptions: {
                    charset: db.charset
                },
                pool: db.pool,
                define: {
                    charset: db.charset,
                    collate: `${db.charset}_general_ci`
                },
                logging: (msg, ...extras) => this.logger.verbose('sql', msg, ...extras)
            });
        });
    }
    sync() {
        return __awaiter(this, void 0, void 0, function* () {
            let recreate = (this.ormMeta.recreate) || false;
            if (recreate) {
                if ((process.env.NODE_ENV || '') == 'production')
                    throw new Error('Server launched with config value orm.recreate enabled. As a security feature, this causes a crash when NODE_ENV = production.');
                this.logger.warn('orm', `Warning: recreating database tables. Note: this option should not be enabled in production.`);
            }
            return yield this.sql.sync({ force: recreate });
        });
    }
    define(modelName, attributes, options) {
        return this.sql.define(modelName, attributes, options);
    }
    get currentTransaction() {
        return this.namespace.get('transaction');
    }
    set currentTransaction(val) {
        this.namespace.set('transaction', val);
    }
    transaction(transactionName, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            let parentTransaction = transaction || this.currentTransaction;
            let sqlTransact = parentTransaction && parentTransaction.sync();
            if (!sqlTransact)
                sqlTransact = yield this.sql.transaction();
            else
                sqlTransact = yield this.sql.transaction({ transaction: sqlTransact });
            let t = new transaction_impl_1.TransactionImpl(transactionName, sqlTransact, parentTransaction);
            this.currentTransaction = t;
            return t;
        });
    }
};
Sequelize = __decorate([
    injectable_decorator_1.Injectable(),
    __metadata("design:paramtypes", [orm_1.OrmMetadata,
        logger_1.Logger,
        cls_namespace_service_1.ClsNamespaceService])
], Sequelize);
exports.Sequelize = Sequelize;
//# sourceMappingURL=sequelize.js.map