"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const createExpressApp = require("express");
const bodyParser = require("body-parser");
const injector_1 = require("../core/injector");
const injectable_decorator_1 = require("../decorators/services/injectable.decorator");
const server_1 = require("../metadata/server/server");
const reflector_1 = require("../orm/reflector");
const reflector_2 = require("../services/reflector");
const logger_1 = require("../services/logger");
const reflector_3 = require("../router/reflector");
const wrap_promise_1 = require("../util/wrap-promise");
const static_middleware_1 = require("./static-middleware");
const http = require("http");
const https = require("https");
const debug_module = require("debug");
let debug = debug_module('express:server');
let Server = Server_1 = class Server {
    constructor(meta) {
        this.errorCode = 0;
        this._logger = new logger_1.Logger(meta.name || null, meta.logLevel);
        this._injector = new injector_1.Injector(this._logger);
        this._injector.provide({ provide: Server_1, useValue: this });
        this._meta = new server_1.ServerMetadata(meta, this._injector);
        for (let q = 0; q < this.meta.inject.length; q++) {
            this._injector.provide(this.meta.inject[q]);
        }
        if (!this._meta.router)
            this._injector.provide({ provide: reflector_3.RouterReflector, useValue: null });
    }
    get meta() {
        return this._meta;
    }
    get logger() {
        return this._logger;
    }
    get app() {
        return this._app;
    }
    get injector() {
        return this._injector;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.logger.info('miter', `Initializing miter server...`);
                if (this.meta.router)
                    yield this.createExpressApp();
                yield this.reflectOrm();
                yield this.startServices();
                if (this.meta.router)
                    this.reflectRoutes();
            }
            catch (e) {
                this.logger.error('miter', `FATAL ERROR: Failed to launch server.`);
                this.logger.error('miter', e);
                return;
            }
            if (this.meta.router)
                this.listen();
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                try {
                    this.logger.info('miter', `Shutting down miter server...`);
                    if (this.meta.router)
                        yield this.stopListening();
                    yield this.stopServices();
                }
                finally {
                    this.logger.ShutDown();
                }
            }
            catch (e) {
                this.logger.error('miter', `FATAL ERROR: Failed to gracefully shutdown server.`);
                this.logger.error('miter', e);
            }
        });
    }
    createExpressApp() {
        this._app = createExpressApp();
        this._app.use(bodyParser.urlencoded({ extended: true }), bodyParser.json());
        if (this.meta.allowCrossOrigin) {
            this.logger.warn('miter', `Server starting with cross-origin policy enabled. This should not be enabled in production.`);
            this._app.use(function (req, res, next) {
                res.header("Access-Control-Allow-Origin", "*");
                res.header("Access-Control-Allow-Headers", req.header("Access-Control-Request-Headers"));
                next();
            });
        }
        this._app.use(static_middleware_1.monkeypatchRequest);
        if (this.meta.router && this.meta.router.middleware && this.meta.router.middleware.length) {
            this._app.use(...this.meta.router.middleware);
        }
    }
    reflectOrm() {
        return __awaiter(this, void 0, void 0, function* () {
            let orm = this.meta.orm;
            if (orm && (typeof orm.enabled === 'undefined' || orm.enabled) && orm.db) {
                this.logger.verbose('orm', `Initializing ORM...`);
                this.ormReflector = this._injector.resolveInjectable(reflector_1.OrmReflector);
                yield this.ormReflector.init();
                this.logger.info('orm', `Finished initializing ORM.`);
            }
            else if (this.meta.orm.models.length) {
                this.logger.warn('orm', `Models included in server metadata, but no orm configuration defined.`);
            }
        });
    }
    startServices() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.verbose('services', `Starting services...`);
            this.serviceReflector = this._injector.resolveInjectable(reflector_2.ServiceReflector);
            yield this.serviceReflector.reflectServices(this.meta.services || []);
            this.logger.info('services', `Finished starting services.`);
        });
    }
    stopServices() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.verbose('services', `Shutting down services...`);
            yield this.serviceReflector.shutdownServices();
            this.logger.info('services', `Finished shutting down services.`);
        });
    }
    reflectRoutes() {
        this.logger.verbose('router', `Loading routes...`);
        this.routerReflector = this._injector.resolveInjectable(reflector_3.RouterReflector);
        this.routerReflector.reflectRoutes();
        this.app.use(this.routerReflector.router.expressRouter);
        this.logger.info('router', `Finished loading routes.`);
    }
    listen() {
        this.logger.info('miter', `Serving`);
        if (this.meta.ssl.enabled) {
            this.webServer = https.createServer({ key: this._meta.ssl.privateKey, cert: this._meta.ssl.certificate }, this.app);
        }
        else {
            this.webServer = http.createServer(this.app);
        }
        this.webServer.listen(this.meta.port, () => this.onListening());
        this.webServer.on("error", (err) => this.onError(err));
    }
    stopListening() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.verbose('miter', `Closing HTTP server...`);
            yield wrap_promise_1.wrapPromise((cb) => {
                if (!this.webServer)
                    return cb();
                this.webServer.close(cb);
            });
            this.logger.info('miter', `Finished closing HTTP server.`);
        });
    }
    onError(error) {
        if (error.syscall !== "listen") {
            throw error;
        }
        let bind = (typeof this.meta.port === "string") ? `pipe ${this.meta.port}` : `port ${this.meta.port}`;
        switch (error.code) {
            case "EACCES":
                this.logger.error('miter', `${bind} requires elevated privileges`);
                this.errorCode = 1;
                this.webServer = undefined;
                this.shutdown();
                break;
            case "EADDRINUSE":
                this.logger.error('miter', `${bind} is already in use`);
                this.errorCode = 1;
                this.webServer = undefined;
                this.shutdown();
                break;
            default:
                throw error;
        }
    }
    onListening() {
        if (!this.webServer)
            throw new Error(`onListening called, but there is no httpServer!`);
        let addr = this.webServer.address();
        let bind = (typeof addr === "string") ? `pipe ${addr}` : `port ${addr.port}`;
        this.logger.info('miter', `Listening on ${bind}`);
    }
};
Server = Server_1 = __decorate([
    injectable_decorator_1.Injectable(),
    __metadata("design:paramtypes", [Object])
], Server);
exports.Server = Server;
var Server_1;
//# sourceMappingURL=server.js.map